/*
 * g2dlp.cpp
 *
 *  Created on: Aug 11, 2015
 *      Author: dave
 */

#include <map>
#include <vector>
#include <array>
#include <algorithm>
#include <string>

#include "misc.h"


using namespace tst;
using namespace std;

long long unsigned no_paths;

#include <list>
//! Partitions list type alias
using parts_list = vector<vector<int>>;

//! Generates all the partitions permutations
//! @param n
//! 	Integer value to partition.
//! @param m
//! 	Partition sequence length.
//! @param parts
//!		Output partitions list.
static inline void partition(int n, int m, parts_list &parts)
{
	vector<int> a(m);	// Working sequence

	first_partition(n, a.begin(), a.end());
	do
	{
		do
		{
			parts.push_back(vector<int>(a));	// Copy
		} while (next_permute(a.begin(), a.end()));
	} while (next_partition(a.begin(), a.end()));
}

//! Alternate merge the two partition lists
//! @param step1
//! 	Step type contained in the parts1 list
//! @param step2
//! 	Step type contained in the parts2 list
//! @param parts1
//! 	First partition list.
//! @param parts1
//! 	Second partition list.
static void altmerge(char step1, char step2,
		parts_list &parts1, parts_list &parts2)
{
	for (auto p1 = parts1.begin(); p1 != parts1.end(); p1++)
	{
		for (auto p2 = parts2.begin(); p2 != parts2.end(); p2++)
		{
			string s;
			s.reserve(64);
			unsigned n = p1->size()+p2->size();
			for (unsigned i = 0; i < n; i++)
			{
#if 1
				auto q = i/2;
				if (i % 2 == 0)
					s.insert(s.end(), (*p1)[q], step1);
				else
					s.insert(s.end(), (*p2)[q], step2);
#endif
			}
			//cout << s << endl;
			no_paths++;
		}
	}
}

void tst::g2dlp(int n1, int n2, int turns)
{
	int k = (turns+1)/2;
	parts_list parts_a, parts_b, parts_c, parts_d;

	if (turns % 2 == 0)
	{
		// First step is Eastward
		if (k+1 <= n1 && k <= n2)
		{
			partition(n1, k+1, parts_a);
			partition(n2, k,   parts_b);
		}
		// First step is Northward
		if (k+1 <= n2 && k <= n1)
		{
			partition(n2, k+1, parts_c);
			partition(n1, k,   parts_d);
		}
	}
	else
	{
		if (k <= n1 && k <= n2)
		{
			//	First step is Eastward
			partition(n1, k, parts_a);
			partition(n2, k, parts_b);
			//	First step is Northward
			parts_c = parts_b;
			parts_d = parts_a;
		}
	}

	altmerge('E', 'N', parts_a, parts_b);
	altmerge('N', 'E', parts_c, parts_d);
}



