/*
 * test_brute.cpp
 *
 *  Created on: Aug 12, 2015
 *      Author: dave
 */


#include "misc.h"
#include "timer.h"
#include <vector>

using namespace tst;
using namespace std;

static int n1_min, n1_max;
static int n2_min, n2_max;


static int parse_args(int argc, char **argv)
{
	switch (argc)
	{
		case 3:
			n1_min = n1_max = atoi(argv[1]);
			n2_min = n2_max = atoi(argv[2]);
			break;
		case 5:
			n1_min = atoi(argv[1]);
			n2_min = atoi(argv[2]);
			n1_max = atoi(argv[3]);
			n2_max = atoi(argv[4]);
			break;
		default:
			printf("\nUsage: %s n1 n2 [n1-max n2-max]\n\n"
					"\tn1: (n1-min) lattice x-size\n"
					"\tn2: (n2-min) lattice y-size\n"
					"\tn1-max: maximum lattice x-size, starting from n1-min\n"
					"\tn2-max: maximum lattice y-size, starting from n2-min\n\n",
					argv[0]);
			return -1;
	}

	// adjust
	if (n1_min < 1)
		n1_min = 1;
	if (n1_max < n1_min)
		n1_max = n1_min;
	if (n2_min < 1)
		n2_min = 1;
	if (n2_max < n2_min)
		n2_max = n2_min;

	printf("\nBRUTE-FORCE\n");
	if (argc == 4)
	{
		printf("\tn1: %d\n", n1_min);
		printf("\tn2: %d\n", n2_min);
	}
	else
	{
		printf("\tn1-min: %d\n", n1_min);
		printf("\tn2-min: %d\n", n2_min);
		printf("\tn1-max: %d\n", n1_max);
		printf("\tn2-max: %d\n", n2_max);
	}
	printf("\n");

	return 0;
}


long long n_paths;
long long n_visits;
int n_turns;

template <typename T>
void visit_turns_check(const T begin, const T end)
{
	n_visits++;
#if 0
	int curr_turns = 0;
	for (auto i = begin; i != end-1; i++)
	{
		if (*i != *(i+1))
			curr_turns++;
	}
	if (curr_turns == n_turns)
		n_paths++;
#endif
	//visit_show(begin, end);
}


int main(int argc, char **argv)
{
	FILE *fp = fopen("test_brute_all.csv", "w");

	if (parse_args(argc, argv) != 0)
		return 1;

	// eventually fills the cache
	bool first = true;
restart:

	for (int n1 = n1_min; n1 <= n1_max; n1++)
	{
		for (int n2 = n2_min; n2 <= n2_max; n2++)
		//int n2 = n1;
		{
			vector<int> a(n1+n2);

			for (int i = 0; i < n1; i++)
				a[i] = 0;
			for (int i = n1; i < n1+n2; i++)
				a[i] = 1;


			n_paths = 0;
			n_visits = 0;
			n_turns = 1;

			timer t;
			auto start = t.get_ticks();
			visit_permutations(a.begin(), a.end(), visit_turns_check);
			auto stop = t.get_ticks();
			double delta = (stop - start) / 1000000.0;

            if (first)
            {
            	first = false;
				goto restart;
			}


			// print to file
			fprintf(fp, "%d,%d,%d,%lld,%lld,%f\n",
					n1, n2, n_turns, n_paths, n_visits, delta);

			// print to standard output
			printf("> n1: %d, n2: %d, turns: %d, paths: %lld, visits: %lld, time: %f\n\n",
					n1, n2, n_turns, n_paths, n_visits, delta);
		}
	}
	fclose(fp);

	return 0;
}

