/*
 * test_g2dlp.cpp
 *
 *  Created on: Aug 14, 2015
 *      Author: dave
 */


#include "misc.h"
#include "timer.h"
#include <vector>


using namespace tst;
using namespace std;


static int n1_min, n1_max;
static int n2_min, n2_max;
static int t_min, t_max;


extern long long unsigned no_paths;



static int parse_args(int argc, char **argv)
{
	switch (argc)
	{
		case 4:
			n1_min = n1_max = atoi(argv[1]);
			n2_min = n2_max = atoi(argv[2]);
			t_min  = t_max  = atoi(argv[3]);
			break;
		case 7:
			n1_min = atoi(argv[1]);
			n2_min = atoi(argv[2]);
			t_min  = atoi(argv[3]);
			n1_max = atoi(argv[4]);
			n2_max = atoi(argv[5]);
			t_max  = atoi(argv[6]);
			break;
		default:
			printf("\nUsage: %s n1 n2 t [n1-max n2-max t-max]\n\n"
					"\tn1: (n1-min) lattice x-size\n"
					"\tn2: (n2-min) lattice y-size\n"
					"\tt:  (t-min) number of turns\n"
					"\tn1-max: maximum lattice x-size, starting from n1-min\n"
					"\tn2-max: maximum lattice y-size, starting from n2-min\n"
					"\tt-max: maximum number of turns, starting from t-min\n\n",
					argv[0]);
			return -1;
	}

	// adjust
	if (n1_min < 1)
		n1_min = 1;
	if (n1_max < n1_min)
		n1_max = n1_min;
	if (n2_min < 1)
		n2_min = 1;
	if (n2_max < n2_min)
		n2_max = n2_min;
	if (t_min < 1)
		t_min = 1;
	if (t_max < t_min)
		t_max = t_min;

	printf("\nG2DLP\n");
	if (argc == 4)
	{
		printf("\tn1: %d\n", n1_min);
		printf("\tn2: %d\n", n2_min);
		printf("\tt:  %d\n", t_min);
	}
	else
	{
		printf("\tn1-min: %d\n", n1_min);
		printf("\tn2-min: %d\n", n2_min);
		printf("\tt-min:  %d\n", t_min);
		printf("\tn1-max: %d\n", n1_max);
		printf("\tn2-max: %d\n", n2_max);
		printf("\tt-max:  %d\n", t_max);
	}
	printf("\n");

	return 0;
}


int main(int argc, char **argv)
{

	FILE *fp = fopen("test_g2dlp.csv", "w");

	if (parse_args(argc, argv) != 0)
		return 1;

	// eventually fills the cache
	int countdown = 1;
restart:

	for (int n1 = n1_min; n1 <= n1_max; n1++)
	{
		for (int n2 = n2_min; n2 <= n2_max; n2++)
		{
			for(int t = t_min; t <= t_max; t++)
			{
				no_paths = 0;

				timer tm;
				auto start = tm.get_ticks();
				g2dlp(n1, n2, t);
				auto stop = tm.get_ticks();
				double delta = (stop - start) / 1000000.0;

				if (countdown)
				{
					countdown--;
					goto restart;
				}

				// print to file
				fprintf(fp, "%d,%d,%d,%lld,%f\n",
						n1, n2, t, no_paths, delta);
				// print to standard output
				printf("> n1: %d, n2: %d, turns: %d, paths: %lld, time: %f\n\n",
						n1, n2, t, no_paths, delta);
			}
		}
	}

	fclose(fp);
}
