/*
 * g2dlp_test.c
 *
 *  Created on: 21/mag/2015
 *      Author: dgalassi
 */


#include "misc.h"
#include "timer.h"

using namespace tst;
using namespace std;

static int n1_min, n1_max;
static int n2_min, n2_max;

extern long long unsigned no_paths;


static int parse_args(int argc, char **argv)
{
	switch (argc)
	{
		case 3:
			n1_min = n1_max = atoi(argv[1]);
			n2_min = n2_max = atoi(argv[2]);
			break;
		case 5:
			n1_min = atoi(argv[1]);
			n2_min = atoi(argv[2]);
			n1_max = atoi(argv[3]);
			n2_max = atoi(argv[4]);
			break;
		default:
			printf("\nUsage: %s n1 n2 [n1-max n2-max]\n\n"
					"\tn1: (n1-min) lattice x-size\n"
					"\tn2: (n2-min) lattice y-size\n"
					"\tn1-max: maximum lattice x-size, starting from n1-min\n"
					"\tn2-max: maximum lattice y-size, starting from n2-min\n\n",
					argv[0]);
			return -1;
	}

	// adjust
	if (n1_min < 1)
		n1_min = 1;
	if (n1_max < n1_min)
		n1_max = n1_min;
	if (n2_min < 1)
		n2_min = 1;
	if (n2_max < n2_min)
		n2_max = n2_min;

	printf("\nG2DLP ALL\n");
	if (argc == 4)
	{
		printf("\tn1: %d\n", n1_min);
		printf("\tn2: %d\n", n2_min);
	}
	else
	{
		printf("\tn1-min: %d\n", n1_min);
		printf("\tn2-min: %d\n", n2_min);
		printf("\tn1-max: %d\n", n1_max);
		printf("\tn2-max: %d\n", n2_max);
	}
	printf("\n");

	return 0;
}


int main(int argc, char **argv)
{
	long long tot_paths;
	double tot_time;

	if (parse_args(argc, argv) != 0)
		return 1;

	FILE *fp = fopen("test_g2dlp_all.csv", "w");

	bool is_first = true;

restart:
	for (int n1 = n1_min; n1 <= n1_max; n1++)
	{
		for (int n2 = n2_min; n2 <= n2_max; n2++)
		{
			tot_time = 0;
			tot_paths = 0;
			int turns = 1;
			while (1)
			{
				no_paths = 0;
				timer tm;
				auto start = tm.get_ticks();
				g2dlp(n1, n2, turns);
				auto stop = tm.get_ticks();
				double delta = (stop - start) / 1000000.0;

				if (is_first)
				{
					is_first = false;;
					goto restart;
				}

				//cout << "t=" << turns << ", paths=" << no_paths << endl;

				if (no_paths == 0)
					break;

				tot_paths += no_paths;
				tot_time += delta;

				turns++;
			}

			// print to file
			fprintf(fp, "%d,%d,%llu,%f\n",
                        n1, n2, tot_paths, tot_time);
			// print to standard output
			printf("> n1: %d, n2: %d, paths: %llu, time: %f\n\n",
                        n1, n2, tot_paths, tot_time);
		}
	}
	fclose(fp);

	return 0;
}


