/*
 * test_partition.cpp
 *
 *  Created on: Aug 12, 2015
 *      Author: dave
 */


#include "misc.h"
#include "timer.h"


using namespace tst;
using namespace std;


static int n_min, n_max;
static int m_min, m_max;


static int parse_args(int argc, char **argv)
{
	switch (argc)
	{
		case 3:
			n_min = n_max = atoi(argv[1]);
			m_min = m_max = atoi(argv[2]);
			break;
		case 5:
			n_min = atoi(argv[1]);
			m_min = atoi(argv[2]);
			n_max = atoi(argv[3]);
			m_max = atoi(argv[4]);
			break;
		default:
			printf("\nUsage: %s n m [n-max m-max]\n\n"
					"\tn: (n-min) number to partition\n"
					"\tm: (m-min) partition size\n"
					"\tn-max: maximum number to partition, starting from n-min\n"
					"\tm-max: maximum partition size, starting from m-min\n\n",
					argv[0]);
			return -1;
	}

	// adjust
	if (n_min < 1)
		n_min = 1;
	if (n_max < n_min)
		n_max = n_min;
	if (m_min < 1)
		m_min = 1;
	if (m_min > n_min)
		m_min = n_min;
	if (m_max < m_min)
		m_max = m_min;
	if (m_max > n_max)
		m_max = n_max;

	printf("\nPartition H\n");
	if (argc == 3)
	{
		printf("\tn: %d\n", n_min);
		printf("\tm: %d\n", m_min);
	}
	else
	{
		printf("\tn-min: %d\n", n_min);
		printf("\tm-min: %d\n", m_min);
		printf("\tn-max: %d\n", n_max);
		printf("\tm-max: %d\n", m_max);
	}
	printf("\n");

	return 0;
}


long long g_no_partitions;
long long g_no_perms;

template <typename T>
static void visit_count(const T begin, const T end)
{
	g_no_partitions++;
    
    do{
        g_no_perms++;
    } while (next_permute(begin,end));
    	
    //visit_show(begin, end);
}


void partition(int n, int m)
{
	vector<int> v(m);
	first_partition(n, v.begin(), v.end());
	visit_partitions(v.begin(), v.end(), visit_count);
}


long long nparts(int n, int m)
{
    if (m == 1)
        return 1;
    else if (m <= n)
        return nparts(n-1, m-1)+nparts(n-m, m); 
    else
        return 0;

}


int main(int argc, char **argv)
{
	if (parse_args(argc, argv) != 0)
		return 1;

	FILE *fp = fopen("test_partition.csv", "w");

	// eventually fills the cache
	bool first = true;
restart:

	for (int n = n_min; n <= n_max; n++)
	{
		for (int m = m_min; m <= std::min(n, m_max); m++)
		{
            g_no_perms = 0;
			g_no_partitions = 0;

			timer t;
			auto start = t.get_ticks();
			partition(n, m);
			auto stop = t.get_ticks();
			double delta = (stop - start) / 1000000.0;

	        if (first)
	        {
	        	first = false;
				goto restart;
			}

			// print to standard output
            long long theo = nparts(n, m);
            if (theo != g_no_partitions) 
            {
                printf(">> n: %d, m: %d\n", n, m);
                printf(">> theo: %lld\n", theo);
                printf(">> comp: %lld\n", g_no_partitions);
                return 0;
            }

            // print to file
			fprintf(fp, "%d,%d,%lld,%f,%lld\n", n, m, g_no_partitions, delta, g_no_perms);

            printf("> n: %d, m: %d, partitions: %lld, time: %f, perms: %lld\n\n", 
                n, m, g_no_partitions, delta, g_no_perms);
        }
	}

	return 0;
}
